@@ -2,20 +2,23 @@
 
 from pages.utils import get_placeholders, normalize_url
 from pages.managers import PageManager, ContentManager
-from pages.managers import PageAliasManager
+from pages.managers import PageAliasManager, ISODATE_FORMAT
 from pages import settings
 
 from datetime import datetime
 from django.db import models
-from django.contrib.auth.models import User
+from django.contrib.auth.models import User, SiteProfileNotAvailable
+from django.db.models import Max
 from django.utils.translation import ugettext_lazy as _
 from django.utils.safestring import mark_safe
 from django.core.cache import cache
 from django.core.urlresolvers import reverse
+from django.core.exceptions import ObjectDoesNotExist
 from django.contrib.sites.models import Site
 
 from mptt.models import MPTTModel
-from taggit.managers import TaggableManager
+if settings.PAGE_TAGGING:
+    from taggit.managers import TaggableManager
 
 PAGE_CONTENT_DICT_KEY = ContentManager.PAGE_CONTENT_DICT_KEY
 
@@ -102,7 +105,7 @@ class Page(MPTTModel):
     objects = PageManager()
 
     if settings.PAGE_TAGGING:
-        tags = TaggableManager()
+        tags = TaggableManager(blank=True)
 
     class Meta:
         """Make sure the default page ordering is correct."""
@@ -118,6 +121,7 @@ class Page(MPTTModel):
         self._languages = None
         self._complete_slug = None
         self._content_dict = None
+        self._is_first_root = None
         super(Page, self).__init__(*args, **kwargs)
 
     def save(self, *args, **kwargs):
@@ -177,6 +181,7 @@ class Page(MPTTModel):
         """Invalidate cached data for this page."""
 
         cache.delete(self.PAGE_LANGUAGES_KEY % (self.id))
+        cache.delete('PAGE_FIRST_ROOT_ID')
         self._languages = None
         self._complete_slug = None
         self._content_dict = dict()
@@ -218,13 +223,23 @@ class Page(MPTTModel):
         return languages
 
     def is_first_root(self):
-        """Return ``True`` if the page is the first root pages."""
+        """Return ``True`` if this page is the first root pages."""
         if self.parent:
             return False
-        root_pages = Page.objects.root()
-        if len(root_pages):
-            return Page.objects.root()[0].id == self.id
-        return False
+        if self._is_first_root is not None:
+            return self._is_first_root
+        first_root_id = cache.get('PAGE_FIRST_ROOT_ID')
+        if first_root_id is not None:
+            self._is_first_root = first_root_id == self.id
+            return self._is_first_root
+        try:
+            first_root_id = Page.objects.root().values('id')[0]['id']
+        except IndexError:
+            first_root_id = None
+        if first_root_id is not None:
+            cache.set('PAGE_FIRST_ROOT_ID', first_root_id)
+        self._is_first_root = self.id == first_root_id
+        return self._is_first_root
 
     def get_url_path(self, language=None):
         """Return the URL's path component. Add the language prefix if
@@ -232,14 +247,21 @@ class Page(MPTTModel):
 
         :param language: the wanted url language.
         """
+        if self.is_first_root():
+            # this is used to allow users to change URL of the root
+            # page. The language prefix is not usable here.
+            try:
+                return reverse('pages-root')
+            except Exception:
+                pass
+        url = self.get_complete_slug(language)
         if not language:
             language = settings.PAGE_DEFAULT_LANGUAGE
-        url = reverse('pages-root')
-        if url.endswith('//'):
-            url = url[:-1]
         if settings.PAGE_USE_LANGUAGE_PREFIX:
-            url += str(language) + '/'
-        return url + self.get_complete_slug(language)
+            return reverse('pages-details-by-path',
+                args=[language, url])
+        else:
+            return reverse('pages-details-by-path', args=[url])
 
     def get_absolute_url(self, language=None):
         """Alias for `get_url_path`.
@@ -251,7 +273,7 @@ class Page(MPTTModel):
         """
         return self.get_url_path(language=language)
 
-    def get_complete_slug(self, language=None):
+    def get_complete_slug(self, language=None, hideroot=True):
         """Return the complete slug of this page by concatenating
         all parent's slugs.
 
@@ -268,8 +290,8 @@ class Page(MPTTModel):
         elif language in self._complete_slug:
             return self._complete_slug[language]
 
-        if settings.PAGE_HIDE_ROOT_SLUG and self.is_first_root():
-            url = ''
+        if hideroot and settings.PAGE_HIDE_ROOT_SLUG and self.is_first_root():
+            url = u''
         else:
             url = u'%s' % self.slug(language)
         for ancestor in self.get_ancestors(ascending=True):
@@ -333,12 +355,31 @@ class Page(MPTTModel):
         placeholders = get_placeholders(self.get_template())
         exposed_content = []
         for lang in self.get_languages():
-            for p_name in [p.name for p in placeholders]:
-                content = self.get_content(lang, p_name, False)
+            for ctype in [p.name for p in placeholders]:
+                content = self.get_content(lang, ctype, False)
                 if content:
                     exposed_content.append(content)
         return u"\r\n".join(exposed_content)
 
+    def content_by_language(self, language):
+        """
+        Return a list of latest published
+        :class:`Content <pages.models.Content>`
+        for a particluar language.
+
+        :param language: wanted language,
+        """
+        placeholders = get_placeholders(self.get_template())
+        content_list = []
+        for ctype in [p.name for p in placeholders]:
+            try:
+                content = Content.objects.get_content_object(self,
+                    language, ctype)
+                content_list.append(content)
+            except Content.DoesNotExist:
+                pass
+        return content_list
+
     def get_template(self):
         """
         Get the :attr:`template <Page.template>` of this page if
@@ -392,7 +433,7 @@ class Page(MPTTModel):
 
     def slug_with_level(self, language=None):
         """Display the slug of the page prepended with insecable
-        spaces equal to the level of page in the hierarchy."""
+        spaces equal to simluate the level of page in the hierarchy."""
         level = ''
         if self.level:
             for n in range(0, self.level):
@@ -403,14 +444,110 @@ class Page(MPTTModel):
         """Used in the admin menu to create the left margin."""
         return self.level * 2
 
+    def dump_json_data(self):
+        """
+        Return a python dict representation of this page for use as part of
+        a JSON export.
+        """
+        def content_langs_ordered():
+            """
+            Return a list of languages ordered by the page content
+            with the latest creation date in each.  This will be used
+            to maintain the state of the language_up_to_date template
+            tag when a page is restored or imported into another site.
+            """
+            params = {'page': self}
+            if self.freeze_date:
+                params['creation_date__lte'] = self.freeze_date
+            cqs = Content.objects.filter(**params)
+            cqs = cqs.values('language').annotate(latest=Max('creation_date'))
+            return [c['language'] for c in cqs.order_by('latest')]
+        languages = content_langs_ordered()
+
+        def language_content(ctype):
+            return dict(
+                (lang, self.get_content(lang, ctype, language_fallback=False))
+                for lang in languages)
+
+        def placeholder_content():
+            """Return content of each placeholder in each language."""
+            out = {}
+            for p in get_placeholders(self.get_template()):
+                if p.name in ('title', 'slug'):
+                    continue # these were already included
+                out[p.name] = language_content(p.name)
+            return out
+
+        def isoformat(d):
+            return None if d is None else d.strftime(ISODATE_FORMAT)
+
+        def custom_email(user):
+            """Allow a user's profile to return an email for the user."""
+            try:
+                profile = user.get_profile()
+            except (SiteProfileNotAvailable, ObjectDoesNotExist):
+                return user.email
+            get_email = getattr(profile, 'get_email', None)
+            return get_email() if get_email else user.email
+
+        return {
+            'complete_slug': dict(
+                (lang, self.get_complete_slug(lang, hideroot=False))
+                for lang in languages),
+            'title': language_content('title'),
+            'author_email': custom_email(self.author),
+            'creation_date': isoformat(self.creation_date),
+            'publication_date': isoformat(self.publication_date),
+            'publication_end_date': isoformat(self.publication_end_date),
+            'last_modification_date': isoformat(self.last_modification_date),
+            'status': {
+                Page.PUBLISHED: 'published',
+                Page.HIDDEN: 'hidden',
+                Page.DRAFT: 'draft'}[self.status],
+            'template': self.template,
+            'sites': (
+                [site.domain for site in self.sites.all()]
+                if settings.PAGE_USE_SITE_ID else []),
+            'redirect_to_url': self.redirect_to_url,
+            'redirect_to_complete_slug': dict(
+                (lang, self.redirect_to.get_complete_slug(
+                    lang, hideroot=False))
+                for lang in self.redirect_to.get_languages()
+                ) if self.redirect_to is not None else None,
+            'content': placeholder_content(),
+            'content_language_updated_order': languages,
+        }
+
+    def update_redirect_to_from_json(self, redirect_to_complete_slugs):
+        """
+        The second pass of PageManager.create_and_update_from_json_data
+        used to update the redirect_to field.
+
+        Returns a messages list to be appended to the messages from the
+        first pass.
+        """
+        messages = []
+        s = ''
+        for lang, s in redirect_to_complete_slugs.items():
+            r = Page.objects.from_path(s, lang, exclude_drafts=False)
+            if r:
+                self.redirect_to = r
+                self.save()
+                break
+        else:
+            messages.append(_("Could not find page for redirect-to field"
+                " '%s'") % (s,))
+        return messages
+
     def __unicode__(self):
         """Representation of the page, saved or not."""
         if self.id:
+            # without ID a slug cannot be retrieved
             slug = self.slug()
             if slug:
                 return slug
-            return "Page %d" % self.id
-        return "Page without id"
+            return u"Page %d" % self.id
+        return u"Page without id"
 
 
 class Content(models.Model):
@@ -434,13 +571,17 @@ class Content(models.Model):
         verbose_name_plural = _('contents')
 
     def __unicode__(self):
-        return "%s :: %s" % (self.page.slug(), self.body[0:15])
+        return u"%s :: %s" % (self.page.slug(), self.body[0:15])
 
 
+page_alias_keywords = {}
+if settings.PAGE_HIDE_SITES:
+    page_alias_keywords = {'limit_choices_to':{'sites':settings.SITE_ID}}
+
 class PageAlias(models.Model):
     """URL alias for a :class:`Page <pages.models.Page>`"""
     page = models.ForeignKey(Page, null=True, blank=True,
-        verbose_name=_('page'))
+        verbose_name=_('page'), **page_alias_keywords)
     url = models.CharField(max_length=255, unique=True)
     objects = PageAliasManager()
 
@@ -453,4 +594,4 @@ class PageAlias(models.Model):
         super(PageAlias, self).save(*args, **kwargs)
 
     def __unicode__(self):
-        return "%s => %s" % (self.url, self.page.get_complete_slug())
+        return u"%s :: %s" % (self.url, self.page.get_complete_slug())
